#include <stdlib.h>
#include <stdbool.h>

/*************************************/
/* Exercice de colles d'informatique */
/*************************************/

/* Titre : Résolution de 2Sat */

/* Dans cet exercice l'ensemble des variables de la logique propositionnelle est
 * supposé être un sous-ensemble de la forme [|0, n-1|]. */

/* Un littéral est alors la donnée :
 * - d'un entier i de [|0, n-1|], représentant la variable propositionnelle ;
 * - d'un booléen : true représente un littéral positif (p), false représente un
     littéral négatif (¬p) */

struct litteral_s {
    int i;                      /* la variable propositionnelle */
    bool b;                     /* littéral positif ou négatif */
};
typedef struct litteral_s litteral;

/* Une instance du problème 2Sat à m clauses est alors la donnée d'un tableau de
 * taille 2m contenant :
 * - dans sa case d'indice 2i, le premier littéral de la i-ème clause,
 * - dans sa case d'indice 2i+1, le second littéral de la i-ème clause. */

struct deux_sat_s {
    int m ;                     /* le nombre de clauses */
    litteral* contenu ;         /* un tableau de taille 2m */
};
typedef struct deux_sat_s deux_sat;

deux_sat exemple() {
    /* Merci wikipedia pour l'exemple :
     * https://en.wikipedia.org/wiki/2-satisfiability */

    int m = 11;
    litteral* contenu = malloc(sizeof(litteral) * 2 * m);

    /* Clause 1 : x0 ∨ x2 */
    contenu[0].b = true;        /* x0 */
    contenu[0].i = 0;
    contenu[1].b = true;        /* x2 */
    contenu[1].i = 2;

    /* Clause 2 : x0 ∨ ¬x3 */
    contenu[2].b = true;        /* x0 */
    contenu[2].i = 0;
    contenu[3].b = false;       /* ¬x3 */
    contenu[3].i = 3;

    /* Clause 3 : x1 ∨ ¬x3 */
    contenu[4].b = true;        /* x1 */
    contenu[4].i = 1;
    contenu[5].b = false;       /* ¬x3 */
    contenu[5].i = 3;

    /* Clause 4 : x1 ∨ ¬x4 */
    contenu[6].b = true;        /* x1 */
    contenu[6].i = 1;
    contenu[7].b = false;       /* ¬x4 */
    contenu[7].i = 4;

    /* Clause 5 : x2 ∨ ¬x4 */
    contenu[8].b = true;        /* x2 */
    contenu[8].i = 2;
    contenu[9].b = false;       /* ¬x4 */
    contenu[9].i = 4;

    /* Clause 6 : x0 ∨ ¬x5 */
    contenu[10].b = true;        /* x0 */
    contenu[10].i = 0;
    contenu[11].b = false;       /* ¬x5 */
    contenu[11].i = 5;

    /* Clause 7 : x1 ∨ ¬x5 */
    contenu[12].b = true;        /* x1 */
    contenu[12].i = 1;
    contenu[13].b = false;       /* ¬x5 */
    contenu[13].i = 5;

    /* Clause 8 : x2 ∨ ¬x5 */
    contenu[14].b = true;        /* x2 */
    contenu[14].i = 2;
    contenu[15].b = false;       /* ¬x5 */
    contenu[15].i = 5;

    /* Clause 9 : x3 ∨ x6 */
    contenu[16].b = true;       /* x3 */
    contenu[16].i = 3;
    contenu[17].b = true;       /* x6 */
    contenu[17].i = 6;

    /* Clause 10 : x4 ∨ x6 */
    contenu[18].b = true;       /* x4 */
    contenu[18].i = 4;
    contenu[19].b = true;       /* x6 */
    contenu[19].i = 6;

    /* Clause 11 : x5 ∨ x6 */
    contenu[20].b = true;       /* x5 */
    contenu[20].i = 5;
    contenu[21].b = true;       /* x6 */
    contenu[21].i = 6;

    return (deux_sat) {m, contenu};
}

/* Q1. Implémenter un algorithme permettant de résoudre le problème de décision
 * 2Sat. */

/* Q2. Le modifier pour qu'il calcule un modèle */
